@@ -5,7 +5,7 @@ use vars qw( @ISA $VERSION $Country );
 
 @ISA = qw( Exporter );
 
-$VERSION = "0.33";
+$VERSION = "0.34";
 
 $Country = 'US';
 
@@ -99,28 +99,30 @@ Here are the currently known agreements:
 
 =item Most Diner's club is now identified as Discover.  (This supercedes the earlier identification of some Diner's club cards as MasterCard inside the US and Canada.)
 
-=item JCB cards in the 3528-3589 range are identified as Discover inside the US and Canada.
+=item JCB cards in the 3528-3589 range are identified as Discover inside the US and territories.
 
-=item China Union Pay cards are identified as Discover cards outside China.
+=item China Union Pay cards are identified as Discover cards in the US, Mexico and most Caribbean countries.
 
 =back
 
-=head1 NOTE ON INTENDED PURPOSE
+=head1 RECEIPT REQUIREMENTS
 
-This module is for verifying I<real world> B<credit cards>.  It is B<NOT> a
-pedantic implementation of the ISO 7812 standard, a general-purpose LUHN
-implementation, or intended for use with "creditcard-like account numbers".
+Discover requires some cards processed on its network to display "PayPal"
+on receipts instead of "Discover".  The receipt_cardtype() subroutine will
+return "PayPal card" for these cards only, and otherwise the same output as
+cardtype().
 
-=head1 AUTHOR
+Use this for receipt display/printing only.
+
+=head1 ORIGINAL AUTHOR
 
 Jon Orwant
 
 The Perl Journal and MIT Media Lab
 
-orwant@tpj.com
+=head1 MAINTAINER
 
 Current maintainer is Ivan Kohler <ivan-business-creditcard@420.am>.
-Please don't bother Jon with emails about this module.
 
 Lee Lawrence <LeeL@aspin.co.uk>, Neale Banks <neale@lowendale.com.au> and
 Max Becker <Max.Becker@firstgate.com> contributed support for additional card
@@ -132,7 +134,7 @@ types.  Lee also contributed a working test.pl.  Alexandr Ciornii
 
 Copyright (C) 1995,1996,1997 Jon Orwant
 Copyright (C) 2001-2006 Ivan Kohler
-Copyright (C) 2007-2014 Freeside Internet Services, Inc.
+Copyright (C) 2007-2016 Freeside Internet Services, Inc.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself, either Perl version 5.8.8 or,
@@ -194,30 +196,32 @@ sub cardtype {
             && 0+$number;
     }
 
-    return "VISA card" if $number =~ /^4[0-8][\dx]{11}([\dx]{3})?$/o;
+    return "VISA card" if $number =~ /^4[0-8][\dx]{11,17}$/o;
 
     return "MasterCard"
-      if   $number =~ /^5[1-5][\dx]{14}$/o
-      ;# || ( $number =~ /^36[\dx]{12}/ && $Country =~ /^(US|CA)$/oi );
+      if $number =~ /^5[1-5][\dx]{14}$/o
+      || $number =~ /^2 ( 22[1-9] | 2[3-9][\dx] | [3-6][\dx]{2} | 7[0-1][\dx] | 720 ) [\dx]{12}$/xo
+      || $number =~ /^2[2-7]xx[\dx]{12}$/o;
 
     return "American Express card" if $number =~ /^3[47][\dx]{13}$/o;
 
     return "Discover card"
-      if   $number =~ /^30[0-5][\dx]{11}([\dx]{2})?$/o  #diner's: 300-305
-      ||   $number =~ /^3095[\dx]{10}([\dx]{2})?$/o     #diner's: 3095
-      ||   $number =~ /^3[689][\dx]{12}([\dx]{2})?$/o   #diner's: 36 38 and 39
-      ||   $number =~ /^6011[\dx]{12}$/o
-      ||   $number =~ /^64[4-9][\dx]{13}$/o
-      ||   $number =~ /^65[\dx]{14}$/o
-      || ( $number =~ /^62[24-68][\dx]{13}$/o && uc($Country) ne 'CN' ) #CUP
-      || ( $number =~ /^35(2[89]|[3-8][\dx])[\dx]{12}$/o && $Country =~ /^(US|CA)$/oi ); #JCB cards in the 3528-3589 range are identified as Discover inside the US and Canada
+      if   $number =~ /^30[0-5][\dx]{13,16}$/o #diner's: 300-305
+      ||   $number =~ /^3095[\dx]{12}$/o       #diner's: 3095
+      ||   $number =~ /^36[\dx]{12,17}$/o      #diner's: 36
+      ||   $number =~ /^3[89][\dx]{14,17}$/o   #diner's: 38 and 39
+      ||   $number =~ /^6011[\dx]{12,15}$/o
+      ||   $number =~ /^64[4-9][\dx]{13,16}$/o
+      ||   $number =~ /^65[\dx]{14,17}$/o
+      || ( $number =~ /^62[24-68][\dx]{13,16}$/o && $Country =~ /^(US|MX|AI|AG|AW|BS|BB|BM|BQ|VG|KY|CW|DM|DO|GD|GP|JM|MQ|MS|BL|KN|LC|VC|MF|SX|TT|TC)$/oi ) #China Union Pay identified as Discover in US, Mexico and Caribbean
+      || ( $number =~ /^35(2[89]|[3-8][\dx])[\dx]{12,15}$/o && $Country =~ /^(US|PR|VI|MP|PW|GU)$/oi ); #JCB cards in the 3528-3589 range are identified as Discover in US, Puerto Rico, US Virgin Islands, Northern Mariana Islands, Palau and Guam
 
     return "Switch"
       if $number =~ /^49(03(0[2-9]|3[5-9])|11(0[1-2]|7[4-9]|8[1-2])|36[0-9]{2})[\dx]{10}([\dx]{2,3})?$/o
       || $number =~ /^564182[\dx]{10}([\dx]{2,3})?$/o
       || $number =~ /^6(3(33[0-4][0-9])|759[0-9]{2})[\dx]{10}([\dx]{2,3})?$/o;
     #redunant with above, catch 49* that's not Switch
-    return "VISA card" if $number =~ /^4[\dx]{12}([\dx]{3})?$/o;
+    return "VISA card" if $number =~ /^4[\dx]{12-18}$/o;
 
     #return "Diner's Club/Carte Blanche"
     #  if $number =~ /^3(0[0-59]|[68][\dx])[\dx]{11}$/o;
@@ -244,6 +248,21 @@ sub cardtype {
     return "Unknown";
 }
 
+sub receipt_cardtype {
+    # Allow use as a class method
+    shift if UNIVERSAL::isa( $_[0], 'Business::CreditCard' );
+
+    my ($number) = @_;
+
+    $number =~ s/[\s\-]//go;
+    $number =~ s/[x\*\.\_]/x/gio;
+
+    #ref Discover IIN Bulletin Feb 2015_021715
+    return "PayPal card" if $number =~ /^6(01104|506[01]0)[\dx]{10,13}$/o;
+
+    cardtype($number);
+}
+
 sub generate_last_digit {
     # Allow use as a class method
     shift if UNIVERSAL::isa( $_[0], 'Business::CreditCard' );
